﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace howto_sierpinski_pentagon
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // Redraw.
        private void nudDepth_ValueChanged(object sender, EventArgs e)
        {
            picPentagon.Refresh();
        }
        private void picPentagon_Resize(object sender, EventArgs e)
        {
            picPentagon.Refresh();
        }

        // Draw.
        private void picPentagon_Paint(object sender, PaintEventArgs e)
        {
            int depth = (int)nudDepth.Value;
            PointF center = new PointF(
                picPentagon.ClientSize.Width / 2,
                picPentagon.ClientSize.Height / 2);
            float radius = (float)Math.Min(center.X,  center.Y);
            DrawPentagon(depth, e.Graphics, Brushes.Red, center, radius);
        }

        // Scale factor for moving to smaller pentagons.
        private float size_scale = (float)(1.0 / (2.0 * (1 + Math.Cos(Math.PI/180*72))));

        // Draw a sierpinski pentagon.
        private void DrawPentagon(int depth,
            Graphics gr, Brush brush,
            PointF center, float radius)
        {
            // If we are done recursing, draw the pentagon.
            if (depth <= 0)
            {
                // Find the pentagon's corners.
                PointF[] points = GetPentagonPoints(center, radius);
                gr.FillPolygon(brush, points);
                //gr.DrawEllipse(Pens.Blue,
                //    center.X - radius, center.Y - radius,
                //    2 * radius, 2 * radius);
            }
            else
            {
                // Find the smaller pentagons' centers.
                float d = radius - radius * size_scale;
                PointF[] centers = GetPentagonPoints(center, d);

                // Recursively draw the smaller pentagons.
                foreach (PointF point in centers)
                {
                    DrawPentagon(depth - 1, gr, brush, point, radius * size_scale);
                }
            }
        }

        // Find the pentagon's corners.
        private PointF[] GetPentagonPoints(PointF center, float radius)
        {
            PointF[] points = new PointF[5];
            double theta = -Math.PI / 2.0;
            double dtheta = 2.0 * Math.PI / 5.0;
            for (int i = 0; i < 5; i++)
            {
                points[i] = new PointF(
                    center.X + (float)(radius * Math.Cos(theta)),
                    center.Y + (float)(radius * Math.Sin(theta)));
                theta += dtheta;
            }
            return points;
        }
    }
}
